/*
 * Copyright (c) 2020 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "xlua_impl.h"
#include <hlog/adif.h>

/*
 * Expose adif writing API
 */

/* return a string with an ADIF header [-0,+1,e] */
static int xlua_adif_write_header(lua_State *state)
{
	struct buffer buf;
	char tmp[4096];
	int ret;

	xlua_checkstack(state, 0, 1);

	buffer_init_static(&buf, tmp, 0, sizeof(tmp), true);

	ret = adif_write_header(&buf);
	if (ret)
		return luaL_error(state, "failed to serialize string: %s",
				  xstrerror(ret));

	lua_pushlstring(state, buffer_data(&buf), buffer_size(&buf));

	return 1;
}

/* return a string with an ADIF QSO [-1,+1,e] */
static int xlua_adif_write_qso(lua_State *state)
{
	struct buffer buf;
	struct qso *qso;
	char tmp[4096];
	int ret;

	xlua_checkstack(state, 1, 1);

	qso = xlua_toqso(state, -1);
	ASSERT3P(qso, !=, NULL);

	buffer_init_static(&buf, tmp, 0, sizeof(tmp), true);

	ret = adif_write_qso(&buf, qso);
	if (ret)
		return luaL_error(state, "failed to serialize qso: %s",
				  xstrerror(ret));

	lua_pushlstring(state, buffer_data(&buf), buffer_size(&buf));

	return 1;
}

static const luaL_Reg adif_funcs[] = {
	{ "write_header", xlua_adif_write_header },
	{ "write_qso", xlua_adif_write_qso },
	{ NULL, NULL },
};

/* register the adif library [-0,+0,e] */
void xlua_pushlib_adif(lua_State *state)
{
	luaL_newlib(state, adif_funcs);
}
