/*
 * Copyright (c) 2020 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __WRAPPER_H
#define __WRAPPER_H

#define WRAP_C_DATA(name, type, putref, extra_fxns)			\
struct lua_wrapper_##name {						\
	type *ptr;							\
	bool ro;							\
};									\
									\
static inline type *xlua_##name##_getptr(lua_State *state, int idx,	\
					 bool *ro_r)			\
{									\
	struct lua_wrapper_##name *wrapper;				\
									\
	wrapper = luaL_checkudata(state, idx, #type);			\
									\
	if (ro_r)							\
		*ro_r = wrapper->ro;					\
									\
	return wrapper->ptr;						\
}									\
									\
static int xlua_wrapper_##name##_gc(lua_State *state)			\
{									\
	putref(xlua_##name##_getptr(state, -1, NULL));			\
									\
	return 0;							\
}									\
									\
static const luaL_Reg provided_##name##_fxns[] = {			\
	{ "__gc", xlua_wrapper_##name##_gc },				\
	{ NULL, NULL },							\
};									\
									\
/* push [-0,+1,e] */							\
void xlua_push##name(lua_State *state, type *ptr, bool ro)		\
{									\
	struct lua_wrapper_##name *wrapper;				\
									\
	if (!ptr) {							\
		lua_pushnil(state);					\
		return;							\
	}								\
									\
	wrapper = lua_newuserdata(state, sizeof(struct lua_wrapper_##name)); \
	ASSERT3P(wrapper, !=, NULL);					\
									\
	wrapper->ptr = (ptr);						\
	wrapper->ro = ro;						\
									\
	/* get the current metatable */					\
	if (luaL_getmetatable(state, #type) != LUA_TTABLE) {		\
		/* initialize a new metatable */			\
		lua_pop(state, 1);					\
		luaL_newmetatable(state, #type);			\
		luaL_setfuncs(state, provided_##name##_fxns, 0);	\
		if (extra_fxns)						\
			luaL_setfuncs(state, extra_fxns, 0);		\
	}								\
									\
	/* set the metatable */						\
	lua_setmetatable(state, -2);					\
}

#endif
